/**
 * Copyright (C) 2011 Agrosense <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.fleet.model;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.io.Serializable;
import org.openide.nodes.PropertySupport;
import org.openide.nodes.Sheet;
import org.openide.util.Exceptions;

/**
 *
 * @author Wytse Visser
 */
public abstract class Machine implements Serializable {

    private String name;
    private String status;
    private String year;
    private String type;
    protected PropertyChangeSupport propertyChangeSupport = new PropertyChangeSupport(this);

    public Machine() {
    }

    public Machine(String name) {
        this.name = name;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        String oldName = this.name;
        this.name = name;
        propertyChangeSupport.firePropertyChange(PROP_NAME, oldName, name);

    }
    public static final String PROP_NAME = "name";

    public String getStatus() {
        if (status == null) {
            return "Available";
        }
        return status;
    }

    public void setStatus(String status) {
        String oldStatus = this.status;
        this.status = status;
        propertyChangeSupport.firePropertyChange(PROP_STATUS, oldStatus, status);
    }
    public static final String PROP_STATUS = "status";

    public String getType() {
        return type;
    }

    public void setType(String type) {
        String oldType = this.type;
        this.type = type;
        propertyChangeSupport.firePropertyChange(PROP_TYPE, oldType, type);
    }
    public static final String PROP_TYPE = "type";

    public String getYear() {
        return year;
    }

    public void setYear(String year) {
        String oldYear = this.year;
        this.year = year;
        propertyChangeSupport.firePropertyChange(PROP_YEAR, oldYear, year);
    }
    public static final String PROP_YEAR = "year";

    public void addPropertyChangeListener(PropertyChangeListener listener) {
        propertyChangeSupport.addPropertyChangeListener(listener);
    }

    public void removePropertyChangeListener(PropertyChangeListener listener) {
        propertyChangeSupport.removePropertyChangeListener(listener);
    }

    public Sheet createSheet() {
        Sheet sheet = Sheet.createDefault();
        Sheet.Set properties = sheet.get(Sheet.PROPERTIES);
        try {
            properties.put(new MachineProperty(String.class, PROP_NAME, "Name"));
            properties.put(new MachineProperty(String.class, PROP_YEAR, "Year"));
            properties.put(new MachineProperty(String.class, PROP_TYPE, "Type"));
            properties.put(new MachineProperty(String.class, PROP_STATUS, "Status"));
        } catch (NoSuchMethodException ex) {
            Exceptions.printStackTrace(ex);
        }
        return sheet;
    }

    protected class MachineProperty extends PropertySupport.Reflection {

        public MachineProperty(Class valueType, String property, String displayName) throws NoSuchMethodException {
            super(Machine.this, valueType, property);
            setDisplayName(displayName);
        }
    }
}
